"""
 作者：${刘海涛}
 日期：2024-09-06 17:18:59
"""
import numpy as np
import mujoco
import pinocchio as pin
import mujoco.viewer
import time

from mujoco import mjtObj

np.set_printoptions(suppress=True, precision=4, linewidth=np.inf)

# 将 mm_base_and_franka_mujoco.urdf 拖到 MuJoCo 软件中，点击（Save xml）生成的 xml文件，mujoco好像不能直接导入urdf文件
# 跟pinocchio类似，使用类型注释，要不 m 这个根本索引不出来。
m: mujoco.MjModel = mujoco.MjModel.from_xml_path("./mm_base_and_franka_mujoco.xml")
d: mujoco.MjData = mujoco.MjData(m)

print(m.nq)  # 系统广义坐标 16
print(m.nv)  # 系统广义速度 15

base_x = 0.8
base_y = 0.9
base_z = 1.0
trans = np.array([base_x, base_y, base_z])

# 根据CoppeliaSim中的规定，角度采用xyz欧拉角，一定要注意。
xyz_euler_x = 1.1
xyz_euler_y = 1.2
xyz_euler_z = 1.3
xyz_euler = np.array([xyz_euler_x, xyz_euler_y, xyz_euler_z])

rotation_matrix = pin.utils.rotate('x', xyz_euler[0]) @ \
                  pin.utils.rotate('y', xyz_euler[1]) @ \
                  pin.utils.rotate('z', xyz_euler[2])

quat = pin.Quaternion(rotation_matrix)

joint_angles = [trans[0], trans[1], trans[2], quat.w, quat.x, quat.y, quat.z,
                0.1, 0.2, 0.3, 0.4, 0.5, 0.6, 0.7,
                0, 0]

qd = np.zeros(m.nv)
qdd = np.zeros(m.nv)
qd[0] = 0.1
qd[1] = 0.2
qd[2] = 0.3
qd[3] = 0.4
qd[4] = 0.5
qd[5] = 0.6
qd[6] = 0.7
qd[7] = 0.8

print(joint_angles)

# 设置关节位置，为所有关节赋值
d.qpos[:len(joint_angles)] = joint_angles
d.qvel[:len(qd)] = qd
d.qacc[:len(qdd)] = qdd

# 计算正向运动学，更新仿真状态。
mujoco.mj_forward(m, d)



# mujoco中的xml文件，没有pinochhio中利用的urdf文件中的固定关节类型，转换的时候直接给合并了。
# 在mujoco中计算正运动学和雅可比矩阵的时候，好像只有joint和body，想要计算其他位置，没想想办法

# 1、手动转化，直接在python代码中进行，设置偏移量，乘以rotation matrix等方式，如下。
#    Fr3_Link7_id = m.body("fr3_link7").id
#    上述得到的body的id的方式和下面的方式是一样的，只是上面的.id索引不出来，但是可以正常使用
Fr3_Link7_id = mujoco.mj_name2id(m, mujoco.mjtObj.mjOBJ_BODY, "fr3_link7")
Fr3_Link7_pos = d.xpos[Fr3_Link7_id]
Fr3_Link7_rot = d.xmat[Fr3_Link7_id].reshape(3, 3)
# Flange 在 fr3_link7 局部坐标系中的偏移量
Flange_2_link7_offset = np.array([0, 0, 0.107])
# 将局部偏移量转换到世界坐标系
world_hand_offset = np.dot(Fr3_Link7_rot, Flange_2_link7_offset)
# Flange 在世界坐标系中的位置 = fr3_link7 的位置 + 偏移
Flange_pos = Fr3_Link7_pos + world_hand_offset
Flange_rot = Fr3_Link7_rot
print("Flange 的全局位置:", Flange_pos)
print("Flange 的旋转矩阵:\n", Flange_rot)

# 2、修改生成的xml文件，手动添加了site标签。

Flange_site_id = m.site('Flange_site').id
Flange_site_pos = d.site_xpos[Flange_site_id]
Flange_site_rot = d.site_xmat[Flange_site_id].reshape(3, 3)
print("Flange_site 的全局位置:", Flange_site_pos)
print("Flange_site 的旋转矩阵:\n", Flange_site_rot)

Flange_site_jacp = np.zeros((3, m.nv))
Flange_site_jacr = np.zeros((3, m.nv))
mujoco.mj_jacSite(m, d, Flange_site_jacp, Flange_site_jacr, Flange_site_id)
Flange_site_jacobian = np.vstack((Flange_site_jacp, Flange_site_jacr))  # vstack函数将多个数组沿垂直（行）方向堆叠的函数
print("带有floating joint的Flange_site处的雅可比矩阵（相对于world）: \n", Flange_site_jacobian)





# 计算 Joint7 的运动学和雅可比矩阵。这个例子中，link7和joint7是一样的位置
# 目前直接计算link7的信息(fr3_link7)
Link7_id = m.body('fr3_link7').id
Link7_pos = d.xpos[Link7_id]
Link7_rot = d.xmat[Link7_id].reshape(3, 3)
print("Link7 的全局位置:\n", Link7_pos)
print("Link7 的旋转矩阵:\n", Link7_rot)

link7_jacp = np.zeros((3, m.nv))
link7_jacr = np.zeros((3, m.nv))
mujoco.mj_jacBody(m, d, link7_jacp, link7_jacr, Link7_id)
Link7_jacobian = np.vstack((link7_jacp, link7_jacr))
print("带有floating joint的 Link7 的雅可比矩阵（相对于world）: \n", Link7_jacobian)

# 为啥利用mj_jac函数不能得到和mj_jacSite相同的结果，point（倒数第二个的输入参数）应该输入什么？np.array([0.0, 0.0, 0.0])也不对好像
# 原因：下面这个例子就可以了，感觉是因为mj_jac的最后一个输入 int body，所以不能计算Site类型 物体 的雅可比矩阵  2024.9.11
link7_jacp_test = np.zeros((3, m.nv))
link7_jacr_test = np.zeros((3, m.nv))
mujoco.mj_jac(m, d, link7_jacp_test, link7_jacr_test, Link7_pos, Link7_id)
Link7_jacobian_test = np.vstack((link7_jacp_test, link7_jacr_test))
print("带有floating joint的 Link 处的雅可比矩阵test（相对于world）: \n", Link7_jacobian_test)


v_Flange = np.zeros(6)
mujoco.mj_objectVelocity(m,d,mjtObj.mjOBJ_SITE,Flange_site_id,v_Flange,0)
print(v_Flange)
print(Flange_site_jacobian @ qd)

