import sys
import pinocchio as pin
from pinocchio.visualize import MeshcatVisualizer
import numpy as np
# 除了安装对应的库，还需要安装 pip install meshcat
# conda env: Test_MM_FK_Jacobian_env

np.set_printoptions(suppress=True, precision=4, linewidth=np.inf)

# urdf_path = "/home/lht/LHT/Test_MM_FK_Jacobian/Test_MM_FK_Jacobian/mm_base_and_franka_pinocchio.urdf"
urdf_path = "mm_base_and_franka_pinocchio.urdf"

model: pin.Model = pin.buildModelFromUrdf(urdf_path)
visual_model = pin.buildGeomFromUrdf(model, urdf_path, pin.GeometryType.VISUAL)
collision_model = pin.buildGeomFromUrdf(model, urdf_path, pin.GeometryType.COLLISION)

data: pin.Data = model.createData()

viz = MeshcatVisualizer(model, collision_model, visual_model)

try:
    viz.initViewer(open=True)
    viz.loadViewerModel()
except ImportError as error:
    print(error)
    sys.exit(0)


# 原来是planar类型的时候，输入的是x，y的位移、z轴转动角度的cos、sin。
# q = pin.randomConfiguration(model)
# q1 = pin.neutral(model)
# theta = -np.pi / 3
# q = np.array([1, 1, np.cos(theta), np.sin(theta),
#               0, 0, 0, -np.pi / 2, 0, np.pi / 2, np.pi / 4,
#               0, 0])


# urdf中的base相对于世界坐标系是floating类型，输入是xyz的位移，还有旋转对应的四元数；floating关节类型需要7个输入。
# 好像修改floating关节的位置量，并不会影响雅可比矩阵。也就是整机的雅可比矩阵和floating的位置量没关系，和旋转角度有关系。
base_x = 0.8
base_y = 0.9
base_z = 1.0
translation = np.array([base_x, base_y, base_z])

# 根据CoppleiaSim中的规定，角度采用xyz欧拉角，一定要注意。
xyz_euler_x = 1.1
xyz_euler_y = 1.2
xyz_euler_z = 1.3
xyz_euler = np.array([xyz_euler_x, xyz_euler_y, xyz_euler_z])

rotation_matrix = pin.utils.rotate('x', xyz_euler[0]) @ \
                  pin.utils.rotate('y', xyz_euler[1]) @ \
                  pin.utils.rotate('z', xyz_euler[2])

quaternion = pin.Quaternion(rotation_matrix)

print(translation[0])
print(translation[1])
print(translation[2])

print('\n')

print(quaternion.x)
print(quaternion.y)
print(quaternion.z)
print(quaternion.w)
q = np.array([translation[0], translation[1], translation[2], quaternion.x, quaternion.y, quaternion.z, quaternion.w,
              # 0, 0, 0, -np.pi / 2, 0, np.pi / 2, 0,
              0.1, 0.2, 0.3, 0.4, 0.5, 0.6, 0.7,
              0, 0])

# 增加关节速度，加速度，为了验证雅可比矩阵 2024.9.13
print(model.nq)  # 系统广义坐标 16
print(model.nv)  # 系统广义速度 15

qd = np.zeros(model.nv)
qdd = np.zeros(model.nv)
qd[0] = 0.1
qd[1] = 0.2
qd[2] = 0.3
qd[3] = 0.4
qd[4] = 0.5
qd[5] = 0.6
qd[6] = 0.7
qd[7] = 0.8

viz.display(q)

print("q=", q)

# 计算hand_tcp的正运动学
Hand_tcp_Id = model.getFrameId("fr3_hand_tcp")
Flange_Id = model.getFrameId("Flange")  # 法兰末端,no hand,跟matlab对应的时候计算这个就可以。
joint7_Id = model.getJointId("fr3_joint7")

pin.forwardKinematics(model, data, q)
pin.updateFramePlacements(model, data)

pin.computeForwardKinematicsDerivatives(model,data,q,qd,qdd)  #更新速度 2024.9.13
v_Flange_world = pin.getFrameVelocity(model,data,Flange_Id,pin.WORLD)  #2024.9.13
print("v_Flange_world = \n", v_Flange_world)  #2024.9.13

hand_tcp_pose = data.oMf[Hand_tcp_Id]
Flange_pose = data.oMf[Flange_Id]
joint7_pose = data.oMi[joint7_Id]
print("hand_tcp_pose \n", hand_tcp_pose)
print("Flange_pose \n", Flange_pose)
print("joint7_pose \n", joint7_pose)

# 计算hand_tcp的Jacobian
pin.computeJointJacobians(model, data, q)
Hand_tcp_Jaco = pin.getFrameJacobian(model, data, Hand_tcp_Id, pin.LOCAL_WORLD_ALIGNED)
print("Hand_tcp_Jaco \n", Hand_tcp_Jaco)

Flange_Jaco = pin.getFrameJacobian(model, data, Flange_Id, pin.WORLD)
# Flange_Jaco = pin.computeFrameJacobian(model, data, q, Flange_Id, pin.WORLD)  # 用这个直接计算也可以。
print("Flange_Jaco \n", Flange_Jaco)
print("v_Flange_world @ qd \n", (Flange_Jaco @ qd))



# 计算joint7的Jacobian
Joint7_Jaco = pin.getJointJacobian(model, data, joint7_Id, pin.WORLD)
print("Joint7_Jaco \n", Joint7_Jaco)


input("Press Enter to continue...")